/* This is the MPI program computing the value of PI */
#include "mpi.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char *argv[])
{
  int done = 0, n, myid, numprocs, i, rc;
  double PI25DT = 3.141592653589793238462643;
  double mypi, pi, h, sum, x, a, tmp[1000];
  MPI_Status status;
  MPI_Request request[1000];

  MPI_Init(&argc,&argv);
  MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
  MPI_Comm_rank(MPI_COMM_WORLD,&myid);
  if (argc != 2) {
    if (myid == 0) {
      printf("Usage: a.out N\n");
    } 
    MPI_Finalize();
    exit(0);
  }
  n = atoi(argv[1]);

  h   = 1.0 / (double) n;
  sum = 0.0;
  for (i = myid + 1; i <= n; i += numprocs) {
    x = h * ((double)i - 0.5);
    sum += 4.0 / (1.0 + x*x);
  }
  mypi = h * sum;

  if (myid != 0) {
	  MPI_Isend(&mypi, 1, MPI_DOUBLE, 0, 100, MPI_COMM_WORLD, &request[0]);
	  MPI_Wait(&request[0], MPI_STATUS_IGNORE);
  } else {
	  for (i = 1; i<numprocs; i++) {
		  MPI_Irecv(&tmp[i-1], 1, MPI_DOUBLE, i, 100, MPI_COMM_WORLD, &request[i-1]);
	  }
	  MPI_Waitall(numprocs-1, request, MPI_STATUSES_IGNORE);
          
          for (i=1; i<numprocs; i++)
		  mypi += tmp[i-1];
	  
	  pi = mypi;
  }
  
  if (myid == 0) 
    printf("pi is approximately %.16f, Error is %.16f\n", pi, fabs(pi - PI25DT));
  MPI_Finalize();
  return 0;
}
